import 'dart:collection';
import 'dart:convert';
import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:path/path.dart' as path;

class AssetBundleL10nDataSource {
  const AssetBundleL10nDataSource({
    @required this.bundlePath,
    this.manifest = 'AssetManifest.json',
  }) : super();

  /// The base asset bundle dir path declared by [manifest].
  ///
  /// Whenever an asset is accessed via [Locale], this [bundlePath] is prefixed
  /// on the final asset name.
  ///
  /// For example, if your project structure is as follows:
  ///
  /// ```
  /// /app
  ///   - l10n
  ///     - en-US/localizations.json
  ///     - pt-BR/localizations.json
  /// ```
  ///
  /// Then the desired [bundlePath] should be `l10n`.
  /// The assets should also be declared in your [manifest] for this to work
  /// properly.
  final String bundlePath;

  /// From where the namespaced files will be loaded from.
  /// This file should contain a Map where the keys represent the asset's path.
  ///
  /// Defaults to 'AssetManifest.json'.
  final String manifest;

  /// Loads all '.json' localization files declared in [manifest] with
  /// [bundlePath] given a [locale]. They must have been previously declared
  /// in `pubspec.yaml`.
  ///
  /// The end result is a [Map] that contains all the namespaces which are
  /// the file names themselves (case sensitive).
  @override
  Future<Map<String, Map<String, Object>>> load(Locale locale) async {
    assert(locale != null);

    final language = locale.toLanguageTag();
    final bundleLocalePath = path.join(bundlePath, language);

    // TODO: add local cache check (dynamically provided localizations)
    final files = (await _manifestFiles())
        // trailing slash is to guarantee the whole dir matches, otherwise
        // it might allow undesired files
        .where((key) => key.contains('$bundleLocalePath/'))
        .where((key) => path.extension(key) == '.json');

    // TODO: make it case insensitive?
    final namespaces = HashMap<String, Map<String, Object>>();
    for (final file in files) {
      // TODO: make this a lazy eval and let loading be handed concurrently?
      final namespace = path.basenameWithoutExtension(file);
      namespaces[namespace] = await rootBundle
          .loadString(file)
          .then((string) => jsonDecode(string));
    }
    return namespaces;
  }

  Future<Iterable<String>> _manifestFiles() async => await rootBundle
      .loadString(manifest)
      .then<Map<String, dynamic>>((string) => jsonDecode(string))
      .then((map) => map.keys);
}
