import 'package:flutter/widgets.dart';

/// Base internationalization class.
/// This simple class only provides a stateful way of managing any
/// internationalization states while giving access to it on the widget tree.
///
/// Common usage:
///
/// ```dart
/// // read
/// I18n.of(context).currentLocale;
/// // or write
/// I18n.of(context).currentLocale = newLocale;
/// ```
class I18n extends StatefulWidget {
  const I18n({Key key, this.locale, @required this.child})
      : assert(child != null),
        super(key: key);

  /// The initial [Locale].
  ///
  /// Defaults to null.
  final Locale locale;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Attempts to retrieve the closest instance from the tree.
  ///
  /// Can return null if none is found.
  static I18nState of(BuildContext context) {
    final _InheritedI18n widget =
        context.inheritFromWidgetOfExactType(_InheritedI18n);
    return widget?._state;
  }

  @override
  I18nState createState() => I18nState();
}

/// The current [I18n] state.
///
/// It contains all internationalization related properties
///
/// To access it use [I18n.of].
class I18nState extends State<I18n> {
  Locale _currentLocale;

  Locale get locale => _currentLocale;

  set locale(Locale currentLocale) {
    setState(() {
      _currentLocale = currentLocale;
    });
  }

  @override
  void initState() {
    super.initState();

    locale = widget.locale;
  }

  @override
  Widget build(BuildContext context) {
    return _InheritedI18n(this, child: widget.child);
  }
}

class _InheritedI18n extends InheritedWidget {
  const _InheritedI18n(
    this._state, {
    Key key,
    Widget child,
  }) : super(key: key, child: child);

  final I18nState _state;

  @override
  bool updateShouldNotify(_InheritedI18n oldWidget) => true;
}
