import 'package:flutter/widgets.dart';

/// Base localization class. The actual instance is provided by [L10nDelegate]
/// which should be declared at App level.
///
/// Typical usage example
///
/// ```dart
/// L10n.of(context).localize('key')
/// ```
abstract class L10n {
  String localize(
    String key, {
    String context,
    int count,
    Map<String, Object> arguments,
  });

  /// Returns the localized resources object for the widget tree that
  /// encloses the given [context].
  ///
  /// Returns null if no resources object of this instance exists within
  /// the given [context].
  static T of<T extends L10n>(BuildContext context) =>
      Localizations.of(context, T);
}

/// A factory for a set of localized resources of [L10n], to be loaded by a
/// [Localizations] widget.
///
/// - [supported] is a list of the currently supported [Locale]s of this
/// delegate.
abstract class L10nDelegate<T extends L10n> extends LocalizationsDelegate<T> {
  const L10nDelegate({@required this.supported})
      : assert(supported != null),
        super();

  /// The list of supported locales by this delegate.
  ///
  /// A supported locale example:
  /// If `en_US` is given, then both `en` and `en_US` are supported by this
  /// delegate, and it will load the normalized locale accordingly.
  /// Same if `pt` is given, it will support both `pt` and `pt_BR` locales.
  ///
  /// Supported | Given | Loaded
  /// :--|:--|:--
  /// `en_US` | `en`    | `en_US`
  /// `en_US` | `en_US` | `en_US`
  /// `pt`    | `pt`    | `pt`
  /// `pt`    | `pt_BR` | `pt`
  /// 'jp'    | 'pt'    | errors
  final List<Locale> supported;

  @override
  bool isSupported(Locale locale) =>
      supported.contains(locale) ||
      supported.any((l) => l.languageCode == locale.languageCode);

  @override
  bool shouldReload(L10nDelegate<T> old) => old.supported != supported;

  /// Normalizes [locale] in case it is not fully supported, but a shorter
  /// or specific one might be.
  ///
  /// e.g. if this delegate supports ['en_US', 'pt']:
  ///
  /// - Both 'en_US' and 'en' => 'en_US'
  /// - Both 'pt_BR' and 'pt' => 'pt'
  Locale normalizeLocale(Locale locale) {
    if (!supported.contains(locale))
      locale = supported.firstWhere(
        (l) => l.languageCode == locale.languageCode,
        orElse: () => throw Exception('Unsupported locale $locale'),
      );
    return locale;
  }

  @override
  Future<T> load(Locale locale) async => loadL10n(normalizeLocale(locale));

  /// Loads a [L10n] class with a normalized [locale] and the [namespaces]
  /// retrieved by the [dataSource].
  ///
  /// See [normalizeLocale] on how normalization process works.
  Future<T> loadL10n(Locale locale);
}
