import 'dart:convert';
import 'dart:ui';

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:i18nu/i18nu.dart';
import 'package:mockito/mockito.dart';

class MockHandler extends Mock {
  Future<String> handle(String key);
}

void main() {
  const bundlePath = 'bundle/path';
  AssetBundleL10nDataSource dataSource;
  MockHandler handler;

  setUp(() {
    dataSource = const AssetBundleL10nDataSource(bundlePath: bundlePath);
    handler = MockHandler();
    defaultBinaryMessenger.setMockMessageHandler('flutter/assets',
        (message) async {
      final key = utf8.decode(message.buffer.asUint8List());
      final result = await handler.handle(key);
      return utf8.encoder.convert(result).buffer.asByteData();
    });
  });

  tearDown(() {
    defaultBinaryMessenger.setMockMessageHandler('flutter/assets', null);
  });

  test('default #manifest', () {
    expect(dataSource.manifest, 'AssetManifest.json');
  });

  group('#load', () {
    setUp(() {
      when(handler.handle(dataSource.manifest)).thenAnswer((_) async => '''{ 
            "another/asset/path": [""], 
            "$bundlePath/en-US/file1.json": [""], 
            "$bundlePath/en-US/file2.json": [""], 
            "$bundlePath/pt/file1.json": [""], 
            "$bundlePath/pt/file2.json": [""]
          }''');
    });

    tearDown(() {
      rootBundle.evict(dataSource.manifest);
    });

    test('given locale null', () {
      expect(() => dataSource.load(null), throwsAssertionError);
    });

    test('given any locale', () async {
      await expectLater(dataSource.load(const Locale('any')), completes);
      verify(handler.handle(dataSource.manifest)).called(1);
    });

    test('given an unregistered locale', () {
      expect(dataSource.load(const Locale('ar')), completion(isEmpty));
    });

    test('given a supported full locale', () async {
      when(handler.handle(argThat(contains('$bundlePath/'))))
          .thenAnswer((_) async => '{}');

      await expectLater(
        dataSource.load(const Locale('en', 'US')),
        completion(equals(<String, Map<String, Object>>{
          'file1': {},
          'file2': {},
        })),
      );

      verify(handler.handle('$bundlePath/en-US/file1.json')).called(1);
      verify(handler.handle('$bundlePath/en-US/file2.json')).called(1);
    });

    test('given an unsupported long locale', () async {
      await expectLater(
        dataSource.load(const Locale('pt-BR')),
        completion(isEmpty),
      );

      verifyNever(handler.handle(argThat(contains('$bundlePath/pt/'))));
      verifyNever(handler.handle(argThat(contains('$bundlePath/pt-BR/'))));
      verifyNever(handler.handle(argThat(contains('$bundlePath/en-US/'))));
    });

    test('given a supported short locale', () async {
      when(handler.handle(argThat(contains('$bundlePath/'))))
          .thenAnswer((_) async => '{}');

      await expectLater(
        dataSource.load(const Locale('pt')),
        completion(equals(<String, Map<String, Object>>{
          'file1': {},
          'file2': {},
        })),
      );

      verify(handler.handle('$bundlePath/pt/file1.json')).called(1);
      verify(handler.handle('$bundlePath/pt/file2.json')).called(1);
      verifyNever(handler.handle(argThat(contains('$bundlePath/en-US/'))));
    });

    test('given an unsupported short locale', () async {
      await expectLater(
        dataSource.load(const Locale('ar')),
        completion(isEmpty),
      );

      verifyNever(handler.handle(argThat(contains('$bundlePath/ar/'))));
      verifyNever(handler.handle(argThat(contains('$bundlePath/pt/'))));
      verifyNever(handler.handle(argThat(contains('$bundlePath/en-US/'))));
    });

    test('when bundle errors', () async {
      const error = 'Some error';
      defaultBinaryMessenger.setMockMessageHandler(
          'flutter/assets', (message) async => throw error);

      expect(dataSource.load(const Locale('any')), throwsA(error));
    });
  });
}
