import 'package:flutter/cupertino.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:i18nu/i18nu.dart';

void main() {
  BuildContext capturedContext;

  final builder = Builder(
    builder: (context) {
      capturedContext = context;
      return Container();
    },
  );

  setUp(() {
    capturedContext = null;
  });

  test('given child null', () {
    expect(() => I18n(child: null), throwsAssertionError);
  });

  testWidgets('given child', (tester) async {
    final child = Container();
    await tester.pumpWidget(I18n(child: child));
    expect(find.byWidget(child), findsOneWidget);
  });

  group('.of', () {
    testWidgets('when not in hierarchy', (tester) async {
      await tester.pumpWidget(builder);
      expect(I18n.of(capturedContext), isNull);
    });

    testWidgets('when in hierarchy', (tester) async {
      await tester.pumpWidget(I18n(child: builder));
      expect(I18n.of(capturedContext), isNotNull);
    });
  });

  group('#locale', () {
    const locale = Locale('pt', 'BR');

    testWidgets('given initial locale', (tester) async {
      await tester.pumpWidget(I18n(child: builder, locale: locale));
      expect(I18n.of(capturedContext).locale, locale);
    });

    testWidgets('without initial locale', (tester) async {
      await tester.pumpWidget(I18n(child: builder));
      expect(I18n.of(capturedContext).locale, isNull);
    });

    testWidgets('given a new value', (tester) async {
      await tester.pumpWidget(I18n(
        child: Builder(
          builder: expectAsync1(
            (context) {
              capturedContext = context;
              return Container();
            },
            count: 2,
          ),
        ),
      ));

      I18n.of(capturedContext).locale = locale;
      await tester.pump();
      expect(I18n.of(capturedContext).locale, locale);
    });

    testWidgets('given the same value', (tester) async {
      await tester.pumpWidget(I18n(
        locale: locale,
        child: Builder(
          builder: expectAsync1(
            (context) {
              capturedContext = context;
              return Container();
            },
            // still rebuilds
            count: 2,
          ),
        ),
      ));

      I18n.of(capturedContext).locale = locale;
      await tester.pump();
      expect(I18n.of(capturedContext).locale, locale);
    });
  });
}
