import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:i18nu/i18nu.dart';
import 'package:mockito/mockito.dart';

class MockL10n extends Mock implements L10n {}

class ConcreteL10nDelegate extends L10nDelegate {
  ConcreteL10nDelegate({@required List<Locale> supported, this.builder})
      : super(supported: supported);

  Future<L10n> Function(Locale locale) builder;

  @override
  Future<L10n> loadL10n(Locale locale) => builder(locale);
}

void main() {
  const supportedLocales = [Locale('en', 'US'), Locale('pt')];
  ConcreteL10nDelegate delegate;
  L10n l10n;

  setUp(() {
    l10n = MockL10n();
    delegate = ConcreteL10nDelegate(supported: supportedLocales);
  });

  test('given supported null', () {
    expect(
      () => ConcreteL10nDelegate(supported: null),
      throwsAssertionError,
    );
  });

  test('#isSupported', () {
    expect(delegate.isSupported(const Locale('en')), isTrue);
    expect(delegate.isSupported(const Locale('en', 'US')), isTrue);
    expect(delegate.isSupported(const Locale('pt')), isTrue);
    expect(delegate.isSupported(const Locale('pt', 'BR')), isTrue);

    expect(delegate.isSupported(const Locale('ar')), isFalse);
    expect(delegate.isSupported(const Locale('gr', 'US')), isFalse);
  });

  group('#shouldReload', () {
    test('given same', () {
      expect(delegate.shouldReload(delegate), isFalse);
    });

    test('when supported differs', () {
      final another = ConcreteL10nDelegate(supported: []);
      expect(delegate.shouldReload(another), isTrue);
    });
  });

  group('#load', () {
    test('when loadL10n errors', () {
      const error = 'Some error';
      delegate.builder = expectAsync1((_) async => throw error);

      expect(delegate.load(supportedLocales.first), throwsA(error));
    });

    test('when loadL10n succeeds', () {
      delegate.builder = expectAsync1((_) async => l10n);

      expect(delegate.load(supportedLocales.first), completes);
    });

    test('given a supported long locale', () {
      const locale = Locale('en', 'US');
      delegate.builder = expectAsync1((argument) async {
        expect(argument, locale);
        return l10n;
      });

      expect(delegate.load(locale), completes);
    });

    test('given a supported short locale', () {
      const locale = Locale('pt');
      delegate.builder = expectAsync1((argument) async {
        expect(argument, locale);
        return l10n;
      });

      expect(delegate.load(locale), completes);
    });

    test('given a partially supported full locale', () {
      delegate.builder = expectAsync1((argument) async {
        expect(argument, const Locale('pt'));
        return l10n;
      });

      expect(delegate.load(const Locale('pt', 'BR')), completes);
    });

    test('given a partially supported short locale', () {
      delegate.builder = expectAsync1((argument) async {
        expect(argument, const Locale('pt'));
        return l10n;
      });

      expect(delegate.load(const Locale('pt', 'BR')), completes);
    });

    test('given an unsupported locale', () {
      const locale = Locale('jp');
      delegate.builder = expectAsync1((_) async => l10n, count: 0);

      expect(delegate.load(locale), throwsException);
    });
  });
}
